@@ -36,15 +36,15 @@ from wpscan_parser import parse_results
 # Setup configuration: will be parsed by setup.py -------------------
 # Values must be in one line
 # Project version.
-VERSION='0.5.3'
+VERSION='0.5.4'
 # URL that will be displayed in help and other places
 GIT_URL="https://github.com/tristanlatr/WPWatcher"
 # Authors
 AUTHORS="Florian Roth, Tristan Landès"
 
 # How many seconds to wait when API limit reached
-# 86400=24h
-API_WAIT_SLEEP=86400
+# 86400s=24h
+API_WAIT_SLEEP=timedelta(hours=24)
 
 # WPWatcher class ---------------------------------------------------------------------
 class WPWatcher():
@@ -76,21 +76,23 @@ class WPWatcher():
         log.info("Configured WordPress sites: %s"%([s['url'] for s in self.conf['wp_sites']]) )
 
         self.wp_reports=self.build_wp_reports()
-    @staticmethod
-    def find_wp_reports_file(create=False):
+    
+    def find_wp_reports_file(self, create=False):
         wp_reports=None
         if 'APPDATA' in os.environ: 
-            p=os.path.join(os.environ['APPDATA'],'.wpwatcher/wp_reports.json')
-            # if create: os.makedirs(, exist_ok=True)
-            if os.path.isfile(p) or create: wp_reports=p
-        elif 'XDG_CONFIG_HOME' in os.environ: 
-            p=os.path.join(os.environ['XDG_CONFIG_HOME'],'.wpwatcher/wp_reports.json')
+            if self.conf['daemon']: p=os.path.join(os.environ['APPDATA'],'.wpwatcher/wp_reports.daemon.json')
+            else: p=os.path.join(os.environ['APPDATA'],'.wpwatcher/wp_reports.json')
             if os.path.isfile(p) or create: wp_reports=p
         elif 'HOME' in os.environ: 
-            p=os.path.join(os.environ['HOME'],'.wpwatcher/wp_reports.json')
+            if self.conf['daemon']: p=os.path.join(os.environ['HOME'],'.wpwatcher/wp_reports.daemon.json')
+            else: p=os.path.join(os.environ['HOME'],'.wpwatcher/wp_reports.json')
             if os.path.isfile(p) or create: wp_reports=p
-        if os.path.isfile('./wp_reports.json'): 
-            wp_reports='./wp_reports.json'
+        elif 'XDG_CONFIG_HOME' in os.environ: 
+            if self.conf['daemon']: p=os.path.join(os.environ['XDG_CONFIG_HOME'],'.wpwatcher/wp_reports.daemon.json')
+            else: p=os.path.join(os.environ['XDG_CONFIG_HOME'],'.wpwatcher/wp_reports.json')
+            if os.path.isfile(p) or create: wp_reports=p
+        # if os.path.isfile('./wp_reports.json'): 
+        #     wp_reports='./wp_reports.json'
         if create:
             os.makedirs(os.path.join(os.environ['HOME'],'.wpwatcher'), exist_ok=True)
             if not os.path.isfile(wp_reports):
@@ -117,22 +119,21 @@ class WPWatcher():
                             log.info("Failure. Scans aborted.")
                             exit(-1)
                 else:
-                    log.info("The file database file %s do not exist. It will be created at the end of the scans."%(self.conf['wp_reports']))
+                    log.info("The database file %s do not exist. It will be created at the end of the scans."%(self.conf['wp_reports']))
         return wp_reports
 
     def write_wp_reports(self, new_wp_report_list):
+        # Update the sites that have been scanned, keep others
+        # Keep same report order add append new sites at the bottom
+        for newr in new_wp_report_list:
+            new=True
+            for r in self.wp_reports:
+                if r['site']==newr['site']:
+                    self.wp_reports[self.wp_reports.index(r)]=newr
+                    new=False
+                    break
+            if new: self.wp_reports.append(newr)
         if self.conf['wp_reports']!='null':
-            # Update the sites that have been scanned, keep others
-            # Keep same report order add append new sites at the bottom
-            for newr in new_wp_report_list:
-                new=True
-                for r in self.wp_reports:
-                    if r['site']==newr['site']:
-                        self.wp_reports[self.wp_reports.index(r)]=newr
-                        new=False
-                        break
-                if new: self.wp_reports.append(newr)
-                
             try:
                 with open(self.conf['wp_reports'],'w') as reportsfile:
                     json.dump(self.wp_reports, reportsfile, indent=4)
@@ -143,6 +144,7 @@ class WPWatcher():
                 if self.conf['fail_fast']: 
                     log.info("Failure. Scans aborted.")
                     exit(-1)
+        return self.wp_reports
 
     # Replace --api-token param with *** for safe logging
     @staticmethod
@@ -325,13 +327,13 @@ class WPWatcher():
             wp_report['last_email']=last_wp_report['last_email']
                             
     def delay_scans(self, scanned_sites, wp_reports_list):
-        log.info("API limit has been reached, sleeping %s seconds and continuing the scans..."%API_WAIT_SLEEP)
-        self.write_wp_reports(wp_reports_list)
-        time.sleep(API_WAIT_SLEEP)
+        log.info("API limit has been reached, sleeping %s and continuing the scans..."%API_WAIT_SLEEP)
+        wp_reports_db=self.write_wp_reports(wp_reports_list)
+        time.sleep(API_WAIT_SLEEP.total_seconds())
         # Instanciating a new WPWatcher object and continue scans with all the non processed sites
         self.conf['wp_sites']=[s for s in self.conf['wp_sites'] if s['url'] not in scanned_sites]
         delayed=WPWatcher(self.conf)
-        return(delayed.run_scans_and_notify())
+        return(delayed.run_scans_and_notify(wp_reports_db))
 
     # Run WPScan on defined websites
     def run_scans_and_notify(self, last_wp_report_list=None):
@@ -341,6 +343,15 @@ class WPWatcher():
         scanned_sites=[]
         exit_code=0
         for wp_site in self.conf['wp_sites']:
+            # Check if url is present and fail if not  
+            if 'url' not in wp_site or wp_site['url']=="":
+                log.error("Site must have valid a 'url' key: %s" % (str(wp_site)))
+                exit_code=-1
+                # Fail fast
+                if self.conf['fail_fast']: 
+                    log.info("Failure. Scans aborted.") 
+                    exit(-1)
+                continue
             # Init report variables
             wp_report={
                 "site":wp_site['url'],
@@ -354,22 +365,23 @@ class WPWatcher():
                 "fixed":[],
                 "wpscan_output":None # will be deleted
             }
-
-            # Check if url is present and fail if not  
-            if 'url' not in wp_site or wp_site['url']=="":
-                log.error("Site must have valid a 'url' key: %s" % (str(wp_site)))
-                exit_code=-1
-                # Fail fast
-                if self.conf['fail_fast']: 
-                    log.info("Failure. Scans aborted.") 
-                    exit(-1)
-                continue
+            # Find last site result if any
+            last_wp_report=[r for r in last_wp_report_list if r['site']==wp_site['url']]
+            if len(last_wp_report)>0: 
+                last_wp_report=last_wp_report[0]
+                # Skip if the daemon mode is enabled and scan already happend in the last configured `daemon_loop_wait`
+                if ( self.conf['daemon'] and 
+                    datetime.strptime(wp_report['datetime'],'%Y-%m-%dT%H-%M-%S') - datetime.strptime(last_wp_report['datetime'],'%Y-%m-%dT%H-%M-%S') < self.conf['daemon_loop_sleep']):
+                    log.info("Daemon skipping site %s because already scanned in the last %s"%(wp_site['url'] , self.conf['daemon_loop_sleep']))
+                    continue
+            else: last_wp_report=None
+            
             # Read the wp_site dict and assing default values if needed -------------
             if 'email_to' not in wp_site or wp_site['email_to'] is None: wp_site['email_to']=[]
             if 'false_positive_strings' not in wp_site or wp_site['false_positive_strings'] is None: wp_site['false_positive_strings']=[]
             if 'wpscan_args' not in wp_site or wp_site['wpscan_args'] is None: wp_site['wpscan_args']=[]
-            
-            # WPScan arguments
+
+           # WPScan arguments
             wpscan_arguments=self.conf['wpscan_args']+wp_site['wpscan_args']+['--url', wp_site['url']]
             log.info("Scanning site %s"%wp_site['url'] )
             
@@ -410,11 +422,8 @@ class WPWatcher():
                         log.info("Failure. Scans aborted.")
                         exit(-1)
                 else:
-                    # Find last site result if any
-                    last_wp_report=[r for r in last_wp_report_list if r['site']==wp_site['url']]
-                    if len(last_wp_report)>0:
-                        last_wp_report=last_wp_report[0]
-                        # Update report entry
+                    # Update report entry if any
+                    if last_wp_report:
                         self.update_report(wp_report, last_wp_report)
                     
                     # Print WPScan findings ------------------------------------------------------
@@ -488,12 +497,12 @@ class WPWatcher():
             # Save report
             wp_report_list.append(wp_report)
 
-        self.write_wp_reports(wp_report_list)
+        wp_reports_db=self.write_wp_reports(wp_report_list)
         if exit_code == 0:
             log.info("Scans finished successfully.") 
         else:
             log.info("Scans finished with errors.") 
-        return((exit_code, wp_report_list))
+        return((exit_code, wp_reports_db))
 
 # Configuration template -------------------------
 TEMPLATE_FILE="""[wpwatcher]
@@ -538,7 +547,7 @@ smtp_ssl=Yes
 
 # Daemon settings (recommended to use --daemon)
 # daemon=No
-# daemon_loop_sleep=24h
+# daemon_loop_sleep=12h
 
 # Output (-q , -v)
 # log_file=/home/user/.wpwatcher/wpwatcher.log
@@ -812,22 +821,20 @@ def wpwatcher():
     configuration=build_config(args)
     # Create main object
     wpwatcher=WPWatcher(configuration)
-
+    # If daemon lopping
     if wpwatcher.conf['daemon']: 
         log.info("Daemon mode selected, looping for ever...")
         results=None # Keep databse in memory
         while True:
-            # If daemon, run scans for ever
+            # Run scans for ever
             exit_code,results=wpwatcher.run_scans_and_notify(results)
-            log.info("Sleeping %s and scanning again..."%wpwatcher.conf['daemon_loop_sleep'])
+            log.info("Daemon sleeping %s and scanning again..."%wpwatcher.conf['daemon_loop_sleep'])
             time.sleep(wpwatcher.conf['daemon_loop_sleep'].total_seconds())
             wpwatcher=WPWatcher(build_config(args))
+    # Run scans and quit
     else:
-        # Run scans and quit
         exit_code,results=wpwatcher.run_scans_and_notify()
-
         exit(exit_code)
-        
 
 if __name__ == '__main__':
     wpwatcher()
\ No newline at end of file
